@@ -273,6 +273,45 @@ class ShiftVectorComponentsMFMA(ShiftVectorComponents):
     Shift Vector Components d0,1
     """
     def __call__(self, writer, kernel, tP):
+        """
+           if (glvw > vectorwidth * continuousOutput * threadsInCoal)
+               use all thread algorithm
+           else
+               use partial thread algorithm
+        """
+
+        # TODO: use this for non SourceSwap for B?
+        # this part can  support non SourceSwap for B
+        # But let non SourceSwap for B go original shiftptr path
+        if (not kernel["SourceSwap"]) and tP["isB"]:
+            return ""
+
+        # common parameter
+        glvw            = tP["glvw"]
+        numThreadInWave = writer.kernel["WavefrontSize"]
+        vectorWidth     = kernel["VectorWidth"] if (kernel["SourceSwap"] and tP["isA"]) else 1
+
+        # use to handle MatrixInst 4x4
+        matrixInstM     = kernel["MatrixInstM"] * kernel["MatrixInstBM"] if (kernel["MatrixInstM"] == 4) else kernel["MatrixInstM"]
+        matrixInstN     = kernel["MatrixInstN"] * kernel["MatrixInstBN"] if (kernel["MatrixInstN"] == 4) else kernel["MatrixInstN"]
+
+        # unify process for dimension M/N
+        matrixInstCoal  = matrixInstM if tP["isA"] else matrixInstN
+        matrixInstPrep  = matrixInstN if tP["isA"] else matrixInstM
+
+        # unify process for SourceSwap and non-SourceSwap
+        conThInProcDim  = kernel["SourceSwap"] ^ tP["isB"] # continuous threads in processed dimension(Coalesced dimension)
+        numThreadInCoal = matrixInstCoal if conThInProcDim else (numThreadInWave // matrixInstPrep)
+        numContOutCoal  = 1 if conThInProcDim else kernel["MIOutputVectorWidth"]
+        allContOutCoal  = numContOutCoal * vectorWidth
+
+        if (glvw > (allContOutCoal * numThreadInCoal)):
+           return self.ShiftVectorComponentsMFMAAllThread(writer, kernel, tP)
+        else:
+           return self.ShiftVectorComponentsMFMAPartialThread (writer, kernel, tP)
+
+
+    def ShiftVectorComponentsMFMAPartialThread(self, writer, kernel, tP):
         """ when we enable shift ptr with vectorwidth(2), we shift global read on edge block when size % vectorwidth != 0.
             For example if M size == 3 vector width == 2, we want to do global read for [0-1] and [2-3].
             But 3 is not in memory object, so we shift to do global read [0-1] and [1-2].
@@ -295,12 +334,6 @@ class ShiftVectorComponentsMFMA(ShiftVectorComponents):
 
         kStr = ""
 
-        # TODO: use this for non SourceSwap for B?
-        # this part can  support non SourceSwap for B
-        # But let non SourceSwap for B go original shiftptr path
-        if (not kernel["SourceSwap"]) and tP["isB"]:
-            return kStr
-
         # common parameter
         regPerElem      = kernel["MIRegPerOut"]
         glvw            = tP["glvw"]
@@ -477,7 +510,7 @@ class ShiftVectorComponentsMFMA(ShiftVectorComponents):
                             kStr += writer.comment3("shift d%u r=%u mb=%u vw%d"%(tP["idx"], r, mb, vw))
                             kStr += "label_%04u: // r%u mb%u vw%u %s" % (VWBlockLabels[r-1][mb][vw], r, mb, vw, writer.endLine)
                             kStr += inst("s_mov_b32", sgpr(tmpSgpr), (((ob*subMBShapeCoal + bm*MBShapeCoal + tt*WGShapeCoal) // glvw) + vw), "")
-                            kStr += inst("v_cmpx_eq_u32", sgpr(tmpSgpr, writer.laneSGPRCount), vgpr(gbReg), sgpr(tmpSgpr), "is thread in edge glvw region" )
+                            kStr += inst("_v_cmpx_eq_u32", sgpr(tmpSgpr, writer.laneSGPRCount), vgpr(gbReg), sgpr(tmpSgpr), "is thread in edge glvw region" )
                             kStr += inst("v_and_b32", vgpr(tmpVgpr), kernel["WavefrontSize"]-1, vgpr("Serial"), "permute register between threads")
                             kStr += inst("v_lshlrev_b32", vgpr(tmpVgpr), log2(writer.bpr), vgpr(tmpVgpr), "permute register between threads")
 
@@ -537,3 +570,237 @@ class ShiftVectorComponentsMFMA(ShiftVectorComponents):
         writer.vgprPool.checkIn(mbReg)
 
         return kStr
+
+
+    def ShiftVectorComponentsMFMAAllThread(self, writer, kernel, tP):
+        """
+        """
+
+        kStr = ""
+
+        # common parameter
+        glvw            = tP["glvw"]
+        numThreadInWave = writer.kernel["WavefrontSize"]
+        vectorWidth     = kernel["VectorWidth"] if (kernel["SourceSwap"] and tP["isA"]) else 1
+
+        # use to handle MatrixInst 4x4
+        matrixInstM     = kernel["MatrixInstM"] * kernel["MatrixInstBM"] if (kernel["MatrixInstM"] == 4) else kernel["MatrixInstM"]
+        matrixInstN     = kernel["MatrixInstN"] * kernel["MatrixInstBN"] if (kernel["MatrixInstN"] == 4) else kernel["MatrixInstN"]
+        matrixInstBM    = 1 if (kernel["MatrixInstM"] == 4) else kernel["MatrixInstBM"]
+        matrixInstBN    = 1 if (kernel["MatrixInstN"] == 4) else kernel["MatrixInstBN"]
+
+        # unify process for dimension M/N
+        matrixInstCoal  = matrixInstM              if tP["isA"] else matrixInstN
+        matrixInstPrep  = matrixInstN              if tP["isA"] else matrixInstM
+        matrixInstBCoal = matrixInstBM             if tP["isA"] else matrixInstBN
+        matrixInstBPrep = matrixInstBN             if tP["isA"] else matrixInstBM
+        miWaveGroupCoal = kernel["MIWaveGroup"][0] if tP["isA"] else kernel["MIWaveGroup"][1]
+        miWGIdStride    = numThreadInWave          if tP["isA"] else (numThreadInWave * kernel["MIWaveGroup"][0])
+        miWaveTitleCoal = kernel["MIWaveTile"][0]  if tP["isA"] else kernel["MIWaveTile"][1]
+        miWaveTitlePrep = kernel["MIWaveTile"][1]  if tP["isA"] else kernel["MIWaveTile"][0]
+
+        # unify process for SourceSwap and non-SourceSwap
+        conThInProcDim  = kernel["SourceSwap"] ^ tP["isB"] # continuous threads in processed dimension(Coalesced dimension)
+
+        threadInterval  = 1 if conThInProcDim else matrixInstPrep
+        numThreadInCoal = matrixInstCoal if conThInProcDim else (numThreadInWave // matrixInstPrep)
+
+        numContOutCoal  = 1 if conThInProcDim else kernel["MIOutputVectorWidth"]
+        allContOutCoal  = numContOutCoal * vectorWidth
+
+        OutBlocksInMI   = (matrixInstCoal * matrixInstPrep) // numThreadInWave // numContOutCoal
+        OutBlocksInMI   = 1 if conThInProcDim else OutBlocksInMI
+
+        subMBShapeCoal  = (matrixInstCoal * vectorWidth) if conThInProcDim else ((numThreadInWave // matrixInstPrep) * numContOutCoal)
+        MBShapeCoal     = subMBShapeCoal * OutBlocksInMI
+        MIBShapeCoal    = MBShapeCoal * matrixInstBCoal
+        miOuterTTCoal   = miWaveTitleCoal // vectorWidth
+
+        numOutputsPrep  = (matrixInstCoal * matrixInstPrep // numThreadInWave) if conThInProcDim else 1
+        numOutputsPrep  = numOutputsPrep * matrixInstBPrep * miWaveTitlePrep
+
+        # unify process for dimension M/N
+        regStrideCoal = 1                                                                if tP["isA"] else numOutputsPrep
+        regStridePrep = miOuterTTCoal * matrixInstBCoal * OutBlocksInMI * allContOutCoal if tP["isA"] else 1
+
+        # labels for shiftptr
+        shiftLabels = []
+        GLVWBLKLabels = []
+        for i in range(0, glvw): # grvw block
+            r = (i+1) % glvw    # r = [1,2,3,...,glvw-1, 0], the last one shiftLabels[glvw-1] stores for r=0 -> no shift
+            label = writer.getLabelNum("ShiftVectorComponents%u_shift%u" % (tP["idx"], r) )
+            shiftLabels.append(label)
+            subGLVWBLKLabels = []
+            for glvwBlk in range(0, (MIBShapeCoal // glvw) * miOuterTTCoal):
+                label = writer.getLabelNum("ShiftVectorComponents%u_shift%u_glvwblk%u" % (tP["idx"], r, glvwBlk))
+                subGLVWBLKLabels.append(label)
+            GLVWBLKLabels.append(subGLVWBLKLabels)
+
+        # wgMT value
+        wg      = tP["prevWg"] if writer.prefetchAcrossPersistent else tP["wg"]
+        tmpSgpr = writer.getTmpSgpr(writer.laneSGPRCount).idx()
+
+        tmpVgpr = writer.vgprPool.checkOutAligned(2,2)
+        dummy   = writer.vgprPool.checkOut(1)
+        wgMT    = writer.vgprPool.checkOut(1)
+
+
+        # get M size of edge block
+        kStr += writer.comment1("check which macro tile need to shift")
+        mtReg = writer.vgprPool.checkOut(1)
+        kStr += inst("v_mov_b32"    , vgpr(wgMT), sgpr(wg), "")
+        kStr += inst("v_mul_i32_i24", vgpr(wgMT), hex(-kernel[tP["mt"]]), vgpr(wgMT), "wg*MT")
+        kStr += inst("_v_add_co_u32", vgpr(wgMT), writer.vcc, sgpr("SizesFree+%u"%tP["idx"]), vgpr(wgMT), "wgMT = Size - wg*MT")
+        kStr += inst("v_mov_b32"    , vgpr(mtReg), hex(kernel[tP["mt"]]), "MT")
+        kStr += inst("v_cmp_lt_u32" , sgpr(tmpSgpr,writer.laneSGPRCount), vgpr(wgMT), vgpr(mtReg), "wgMT < MT" )
+        kStr += inst("v_cndmask_b32", vgpr(wgMT), vgpr(mtReg), vgpr(wgMT), sgpr(tmpSgpr,writer.laneSGPRCount), "wgMT = (wgMT < MT) ? wgMT : MT" )
+        kStr += writer.endLine
+
+        # identify which wave have to process
+        kStr += writer.comment1("check which wave need to shift")
+        wReg = writer.vgprPool.checkOut(1)
+        sReg = writer.vgprPool.checkOut(1)
+        kStr += vectorStaticDivide(wReg, "Serial", miWGIdStride, tmpVgpr, tmpSgpr)
+        kStr += vectorStaticRemainder(dummy, wReg, wReg, miWaveGroupCoal, tmpVgpr, tmpSgpr)
+        kStr += vectorStaticDivide(sReg, wgMT, MIBShapeCoal, tmpVgpr, tmpSgpr)
+        kStr += vectorStaticRemainder(dummy, sReg, sReg, miWaveGroupCoal, tmpVgpr, tmpSgpr)
+        kStr += inst("v_cmp_eq_u32" , sgpr(tmpSgpr,writer.laneSGPRCount), vgpr(sReg), vgpr(wReg), "wave_id == block_belong_to_wave?" )
+        kStr += inst("v_cndmask_b32", vgpr(wgMT), vgpr(mtReg), vgpr(wgMT), sgpr(tmpSgpr,writer.laneSGPRCount), "wgMT = (wgMT < MT) ? wgMT : MT" )
+        kStr += writer.endLine
+
+        # glveblkid
+        kStr += writer.comment1("get id of which glvw block need to shift")
+        glvwblkidReg = writer.vgprPool.checkOut(1)
+        kStr += inst("v_mul_i32_i24", vgpr(glvwblkidReg), hex(-MIBShapeCoal), vgpr(wReg), "wg * MIB")
+        kStr += inst("_v_add_co_u32", vgpr(glvwblkidReg), writer.vcc, vgpr(glvwblkidReg), vgpr(wgMT), "wgMT = Size - wg*MIB")
+        kStr += vectorStaticDivide(glvwblkidReg, glvwblkidReg, glvw, tmpVgpr, tmpSgpr, "glvw block id")
+        kStr += writer.endLine
+
+        # rReg : reminder of M_size % vectorwidth
+        # decide to jump to block which handle this case, M_size % vector width
+        kStr += writer.comment1("dispatch to differet shift block for shift")
+        rReg = writer.vgprPool.checkOut(1)
+        kStr += vectorStaticRemainder(dummy, rReg, wgMT, glvw, tmpVgpr, tmpSgpr)
+        for r in range(1, glvw):
+            kStr += inst("v_cmp_eq_u32", writer.vcc, vgpr(rReg), hex(r), "wgMT%%GLVW == %u"%r )
+            kStr += inst("s_cbranch_vccnz label_%04u" % shiftLabels[(r-1)], "branch to shift d%u r=%u"%(tP["idx"], r))
+        kStr += inst("s_branch label_%04u" % shiftLabels[glvw-1], "no shifting" )
+        writer.vgprPool.checkIn(rReg)
+
+        # blocks for handle M_size % vector width
+        glvwBlkInMIB = MIBShapeCoal // glvw
+        numRegInGlvwblkCoal = glvw // numThreadInCoal
+        numRegInMIBCoal = MIBShapeCoal // numThreadInCoal
+        for r in range(1, glvw):
+            kStr += writer.comment3("shift d%u shift=%u"%(tP["idx"], r))
+            kStr += "label_%04u:%s" % (shiftLabels[r-1], writer.endLine)
+            for tt in range(0, miOuterTTCoal):
+                for glvwBlk in range(0, glvwBlkInMIB):
+                    label  = glvwBlk + tt * glvwBlkInMIB
+                    target = glvwBlk + tt * glvwBlkInMIB * miWaveGroupCoal
+                    kStr += inst("v_cmp_eq_u32", writer.vcc, vgpr(glvwblkidReg), hex(target), "")
+                    kStr += inst("s_cbranch_vccnz label_%04u" % GLVWBLKLabels[r-1][label], "branch to shift d%u shift%u glvwblk%u" % (tP["idx"], r, target))
+
+        _, arch2acc = writer.AccToArchMapper(kernel)
+
+        permuteIndexReg = writer.vgprPool.checkOut(1)
+        threadIdInCoalReg = writer.vgprPool.checkOut(1)
+        movReg = writer.vgprPool.checkOut(numContOutCoal*numOutputsPrep)
+        kStr += writer.comment3("Tony Reg %d-%d"%(movReg, movReg+15))
+        for shift in range(1, glvw):
+            for tt in range(0, miOuterTTCoal):
+                for glvwBlk in range(0, glvwBlkInMIB):
+                    label = glvwBlk + tt * glvwBlkInMIB
+                    kStr += writer.comment3("shift d%u shift=%u glvwblk=%u"%(tP["idx"], shift, glvwBlk))
+                    kStr += "label_%04u:%s" % (GLVWBLKLabels[shift-1][label], writer.endLine)
+                    kStr += inst("v_and_b32", vgpr(permuteIndexReg), kernel["WavefrontSize"]-1, vgpr("Serial"), "permute register between threads")
+                    kStr += staticMultiply(vgpr(permuteIndexReg), vgpr(permuteIndexReg), writer.bpr, sgpr(tmpSgpr), "permute register between threads")
+
+                    kStr += vectorStaticDivide(threadIdInCoalReg, "Serial", threadInterval, tmpVgpr, tmpSgpr)
+                    kStr += vectorStaticRemainder(dummy, threadIdInCoalReg, threadIdInCoalReg, numThreadInCoal, tmpVgpr, tmpSgpr)
+
+                    for dstMbblkId in range(glvw//(numContOutCoal*numThreadInCoal)):
+                        for dstThreadId in range(numThreadInCoal):
+                            skip = True
+                            copyInstStr = "v_accvgpr_read_b32" if not kernel["MIArchVgpr"] else "v_mov_b32"
+                            for dstContId in range(numContOutCoal):
+                                dst = dstContId + dstThreadId * numContOutCoal + dstMbblkId * numThreadInCoal * numContOutCoal
+                                src = dst + (glvw - shift)
+                                if (src < glvw):
+                                    skip = False
+                                    srcContId   = src % numContOutCoal
+                                    srcThreadId = (src // numContOutCoal) % numThreadInCoal
+                                    srcMbblkId  = src // (numContOutCoal * numThreadInCoal)
+                                    for ot in range(numOutputsPrep):
+                                      movRegId    = movReg + dstContId + ot * numContOutCoal
+                                      srcGpr      = srcContId + srcMbblkId * numContOutCoal + glvwBlk * numRegInGlvwblkCoal + tt * numRegInMIBCoal
+                                      srcGpr      = srcGpr * regStrideCoal + ot * regStridePrep
+                                      srcGpr      = arch2acc[srcGpr]
+                                      srcGprStr   = accvgpr(srcGpr) if not kernel["MIArchVgpr"] else vgpr(srcGpr)
+                                      kStr += inst(copyInstStr, vgpr(movRegId), srcGprStr, "")
+
+                            if not skip:
+                                kStr += inst("s_nop", "1", "v_accvgpr read vgpr after write vgpr: 2 wait states")
+
+                            needWait = False
+                            for dstContId in range(numContOutCoal):
+                                dst = dstContId + dstThreadId * numContOutCoal + dstMbblkId * numThreadInCoal * numContOutCoal
+                                src = dst + (glvw - shift)
+                                if (src < glvw):
+                                    srcContId   = src % numContOutCoal
+                                    srcThreadId = (src // numContOutCoal) % numThreadInCoal
+                                    srcMbblkId  = src // (numContOutCoal * numThreadInCoal)
+                                    srcGpr      = srcContId + srcMbblkId * numContOutCoal
+                                    if dstThreadId != srcThreadId:
+                                        needWait = True
+                                        permuteOffset = (((srcThreadId - dstThreadId) * threadInterval) % kernel["WavefrontSize"]) * 4
+                                        for ot in range(numOutputsPrep):
+                                            movRegId    = movReg + dstContId + ot * numContOutCoal
+                                            kStr += inst("ds_bpermute_b32", vgpr(movRegId), vgpr(permuteIndexReg), vgpr(movRegId), f"offset:{permuteOffset}", "permute edge values")
+
+                            if needWait:
+                                kStr += inst("s_waitcnt", "lgkmcnt(0)", "wait for swizzle operation")
+
+                            if not skip:
+                                kStr += inst("s_mov_b32", sgpr(tmpSgpr), dstThreadId, "which thread need to shfit in this block")
+                                kStr += inst("_v_cmpx_eq_u32", sgpr(tmpSgpr, writer.laneSGPRCount), vgpr(threadIdInCoalReg), sgpr(tmpSgpr), "is thread in edge glvw region" )
+                                kStr += inst("s_nop", "3", "wait for exec mask")
+
+                            copyInstStr = "v_accvgpr_write_b32" if not kernel["MIArchVgpr"] else "v_mov_b32"
+                            for dstContId in range(numContOutCoal):
+                                dst = dstContId + dstThreadId * numContOutCoal + dstMbblkId * numThreadInCoal * numContOutCoal
+                                src = dst + (glvw - shift)
+                                if (src < glvw):
+                                    for ot in range(numOutputsPrep):
+                                        movRegId    = movReg + dstContId + ot * numContOutCoal
+                                        dstGpr      = dstContId + dstMbblkId * numContOutCoal + glvwBlk * numRegInGlvwblkCoal + tt * numRegInMIBCoal
+                                        dstGpr      = dstGpr * regStrideCoal + ot * regStridePrep
+                                        dstGpr      = arch2acc[dstGpr]
+                                        dstGprStr   = accvgpr(dstGpr) if not kernel["MIArchVgpr"] else vgpr(dstGpr)
+                                        kStr += inst(copyInstStr, dstGprStr, vgpr(movRegId), "")
+
+                            if not skip:
+                                all1mask = "0xFFFFFFFF" if (kernel["WavefrontSize"] == 32) else "0xFFFFFFFFFFFFFFFF"
+                                kStr += inst("s_mov_b{}".format(kernel["WavefrontSize"]), sgpr(tmpSgpr, writer.laneSGPRCount), all1mask, "to restore all threads active")
+                                kStr += inst("s_or_saveexec_b{}".format(kernel["WavefrontSize"]), writer.vcc, sgpr(tmpSgpr,writer.laneSGPRCount), "all threads active")
+                                kStr += inst("s_nop", "3", "wait for exec mask")
+
+                    kStr += inst("s_branch label_%04u" % shiftLabels[glvw-1], "done" )
+
+        kStr += "label_%04u: // end shift0%s" % (shiftLabels[glvw-1], writer.endLine)
+
+        writer.vgprPool.checkIn(movReg)
+        writer.vgprPool.checkIn(threadIdInCoalReg)
+        writer.vgprPool.checkIn(permuteIndexReg)
+
+        writer.vgprPool.checkIn(glvwblkidReg)
+        writer.vgprPool.checkIn(sReg)
+        writer.vgprPool.checkIn(wReg)
+        writer.vgprPool.checkIn(mtReg)
+
+        # checkin scratch vgprs
+        writer.vgprPool.checkIn(tmpVgpr)
+        writer.vgprPool.checkIn(dummy)
+        writer.vgprPool.checkIn(wgMT)
+
+        return kStr
