@@ -375,7 +375,7 @@ class ProblemPredicate(Properties.Predicate):
 
         # if bufferload is performed, we output some predication info for host side,
         # to prevent from some extremely large problems from launching and causing bufferload offset limit < 2^32
-        # thoses cases will not satisfy the assertion thus won't use the kernel.
+        # those cases will not satisfy the assertion thus won't use the kernel.
         # See Common.py for more details, we will need four values:
         # TODO - haven't been fully tested for FP16 and BF, need to verify the false-positive
         if 'BufferLoad' in state and state['BufferLoad'] == True:
@@ -388,18 +388,18 @@ class ProblemPredicate(Properties.Predicate):
             subrv['ShiftPtrElemA'] = state['GlobalLoadVectorWidthA'] if MayShiftA else 0
             subrv['DUorMT1'] = state['DepthU'] if TLUB else state['MacroTile1']
             subrv['DUorMT0'] = state['DepthU'] if TLUA else state['MacroTile0']
-            # value is also a dict for better readibility, client side need to handel the serialization
+            # value is also a dict for better readability, client side need to handel the serialization
             rv += [cls('BufferLoadOffsetLimitCheck', value=subrv)]
 
-        # When doing globol write, may need to load matrix C if beta !=0
-        if 'BufferLoad' in state and state['BufferLoad'] == True:
-            rv += [cls('BufferLoadOffsetLimitCheck_Beta', value=state['MacroTile1'])]
-
-        # similiar check is applied for bufferstore,
+        # similar check is applied for bufferstore,
         # for bufferstore offset, test if the bot-right offset < 2^32,
         # it should be StrideA*MT1, so we need to output MT1 and use the StrideA of problem in host-side for predication
         if 'BufferStore' in state and state['BufferStore'] == True:
-            rv += [cls('BufferStoreOffsetLimitCheck', value=state['MacroTile1'])]
+            val = state['MacroTile1']
+            if (1 in state["AssertSizeLessThan"].keys()):
+                # use smaller value if array size assert is enabled
+                val = min(val, state["AssertSizeLessThan"][1] - 1)
+            rv += [cls('BufferStoreOffsetLimitCheck', value=val)]
 
         if '_GlobalAccumulation' in state and state['_GlobalAccumulation'] != None:
             value = globalParameters['MinKForGSU'] * state['GlobalSplitU']
@@ -478,7 +478,7 @@ class Solution:
                 'problemPredicate',
                 'sizeMapping',
                 'debugKernel',
-                'info',
+                'libraryLogicIndex',
                 'index',
                 'ideals',
                 'linearModel']
@@ -506,7 +506,8 @@ class Solution:
         if 'SolutionIndex' in d:
             rv.index = d['SolutionIndex']
 
-        rv.info = cls.ReadOriginalInfo(d)
+        info = cls.ReadOriginalInfo(d)
+        rv.libraryLogicIndex = int(info.get("SolutionIndex", -1))
 
         rv.sizeMapping = SizeMapping.FromOriginalState(d)
         if 'Ideals' in d:
@@ -542,7 +543,7 @@ class Solution:
         self.problemPredicate = ProblemPredicate('TruePred')
         self.sizeMapping = None
         self.debugKernel = False
-        self.info = {}
+        self.libraryLogicIndex = {}
         self.index = None
         self.ideals = {}
 
