@@ -48,6 +48,12 @@ class SingleSolutionLibrary:
     def remapSolutionIndices(self, indexMap):
         pass
 
+class IndexSolutionLibrary(SingleSolutionLibrary):
+    Tag = "Index"
+
+    def state(self):
+        return self.solution.index
+
 
 class PlaceholderLibrary:
     Tag = 'Placeholder'
@@ -82,9 +88,10 @@ class MatchingLibrary:
         propertyKeys = {
             2: Properties.Property("FreeSizeA", index=0),
             3: Properties.Property("FreeSizeB", index=0),
-            #0: Properties.Property("BatchSize", index=0),
             1: Properties.Property("BoundSize", index=0)
         }
+        if distance == "Equality":
+            propertyKeys[0] = Properties.Property("BatchSize", index=0)
 
         properties = list([propertyKeys[i] for i in indices if i in propertyKeys])
         keyOrder = [i for i, j in enumerate(indices) if j in propertyKeys]
@@ -94,10 +101,13 @@ class MatchingLibrary:
         for row in origTable:
             try:
                 index = row[1][0]
-                value = SingleSolutionLibrary(solutions[index])
+                value = IndexSolutionLibrary(solutions[index])
                 key = list([row[0][i] for i in keyOrder])
-                #key = list(row[0][0:len(properties)])
-                entry = {"key": key, "value": value, "speed": row[1][1]}
+                if distance == "GridBased":
+                    entry = {"key": key, "index": value}
+                else:
+                    entry = {"key": key, "index": value, "speed": row[1][1]}
+
                 table.append(entry)
             except KeyError:
                 pass
@@ -130,11 +140,11 @@ class MatchingLibrary:
 
 class DecisionTreeLibrary:
     Tag = "DecisionTree"
-    StateKeys = [("type", "tag"), "properties", "trees"]
+    StateKeys = [("type", "tag"), "features", "trees"]
 
     @classmethod
     def FromOriginalState(cls, d, solutions):
-        properties = d["properties"]
+        features = d["features"]
         origTrees = d["trees"]
 
         trees = []
@@ -146,7 +156,7 @@ class DecisionTreeLibrary:
             entry = {"tree": tree["tree"], "value": value}
             trees.append(entry)
 
-        return cls(properties, trees)
+        return cls(features, trees)
 
     @property
     def tag(self):
@@ -160,8 +170,8 @@ class DecisionTreeLibrary:
     def remapSolutionIndices(self, indexMap):
         pass
 
-    def __init__(self, properties, trees):
-        self.properties = properties
+    def __init__(self, features, trees):
+        self.features = features
         self.trees = trees
 
 
@@ -448,7 +458,7 @@ class MasterSolutionLibrary:
     def merge(self, other, startIndex=0):
         assert self.__class__ == other.__class__
 
-        curIndex = max(startIndex, max(self.solutions.keys()) + 1) if self.solutions else 0
+        curIndex = max(startIndex, max(self.solutions.keys()) + 1 if self.solutions else 0)
         if self.lazyLibraries:
             curIndex = max(
                 curIndex,
