@@ -304,12 +304,12 @@ class SolutionWriter:
       s += "#if HIP_VERSION >= 50220730\n"
       s += "%sint hip_version;\n" % (t)
       s += "%shipRuntimeGetVersion(&hip_version);\n" % (t)
-      s += "%sif(hip_version >= 50220730)\n" (t)
-      s += "%s{\n" (t)
-      s += "%s    hipDeviceGetAttribute(&deviceProperties.multiProcessorCount,\n" (t)
-      s += "%s                          hipDeviceAttributePhysicalMultiProcessorCount,\n" (t)
-      s += "%s                          deviceId);\n" (t)
-      s += "%s}\n" (t)
+      s += "%sif(hip_version >= 50220730)\n" % (t)
+      s += "%s{\n" % (t)
+      s += "%s    hipDeviceGetAttribute(&deviceProperties.multiProcessorCount,\n" % (t)
+      s += "%s                          hipDeviceAttributePhysicalMultiProcessorCount,\n" % (t)
+      s += "%s                          deviceId);\n" % (t)
+      s += "%s}\n" % (t)
       s += "#endif\n"
       s += "%sunsigned int numGroups = totalWorkGroups0 * totalWorkGroups1;\n" % (t)
       s += "%sglobalWorkSize[0][0] = (deviceProperties.multiProcessorCount * %u < numGroups) ? (deviceProperties.multiProcessorCount * %u) : numGroups;\n" \
