@@ -442,6 +442,8 @@ namespace Tensile
                 while(solutionIterator->moreSolutionsInProblem())
                 {
                     auto solution = solutionIterator->getSolution();
+                    if(solution == nullptr)
+                        throw std::runtime_error("Could not find a solution");
 
                     listeners.preSolution(*solution);
 
@@ -584,6 +586,8 @@ int main(int argc, const char* argv[])
             while(solutionIterator->moreSolutionsInProblem())
             {
                 auto solution = solutionIterator->getSolution();
+                if(solution == nullptr)
+                    throw std::runtime_error("Could not find a solution");
 
                 listeners.preSolution(*solution);
 
@@ -614,10 +618,12 @@ int main(int argc, const char* argv[])
                                     HIP_CHECK_EXC(
                                         adapter.launchKernels(kernels, stream, nullptr, nullptr));
                                 listeners.postWarmup();
+                                // Do validation after first warmup
+                                if(i == 0)
+                                    listeners.validateWarmups(
+                                        inputs, warmupStartEvents, warmupStopEvents);
                             }
 
-                            listeners.validateWarmups(inputs, warmupStartEvents, warmupStopEvents);
-
                             size_t syncs = listeners.numSyncs();
                             size_t enq   = listeners.numEnqueuesPerSync();
 
