@@ -392,6 +392,11 @@ namespace Tensile
             rv.args.append<typename TypedInputs::BType const* const*>("batchB", inputs.batchB);
         }
 
+        rv.args.append<uint64_t>("offsetD", d.offset());
+        rv.args.append<uint64_t>("offsetC", c.offset());
+        rv.args.append<uint64_t>("offsetA", a.offset());
+        rv.args.append<uint64_t>("offsetB", b.offset());
+
         rv.args.append<typename TypedInputs::AlphaType>("alpha", inputs.alpha);
         if(std::is_same<typename TypedInputs::AlphaType, Half>::value && !isSourceKernel())
             rv.args.append<typename TypedInputs::AlphaType>("alpha_2", inputs.alpha);
@@ -579,11 +584,6 @@ namespace Tensile
             rv.args.append<uint32_t>("magicNumberWgmRemainder1", magicNumberWgmRemainder1);
         }
 
-        rv.args.append<uint32_t>("offsetD", d.offset());
-        rv.args.append<uint32_t>("offsetC", c.offset());
-        rv.args.append<uint32_t>("offsetA", a.offset());
-        rv.args.append<uint32_t>("offsetB", b.offset());
-
         if(!isSourceKernel())
         {
             rv.args.append<uint32_t>("pad", 0);
@@ -649,6 +649,9 @@ namespace Tensile
         else
             rv.args.append<typename TypedInputs::CType const* const*>("batchC", inputs.batchC);
 
+        rv.args.append<uint64_t>("offsetD", d.offset());
+        rv.args.append<uint64_t>("offsetC", c.offset());
+
         if(sizeMapping.globalAccumulation)
         {
             size_t stride = d.sizes()[0];
@@ -677,9 +680,6 @@ namespace Tensile
             idx++;
         }
 
-        rv.args.append<uint32_t>("offsetD", d.offset());
-        rv.args.append<uint32_t>("offsetC", c.offset());
-
         rv.args.append<typename TypedInputs::BetaType>("beta", inputs.beta);
 
         //Pass along code object dependency
@@ -758,6 +758,9 @@ namespace Tensile
         else
             rv.args.append<typename TypedInputs::CType const* const*>("batchC", inputs.batchC);
 
+        rv.args.append<uint64_t>("offsetD", d.offset());
+        rv.args.append<uint64_t>("offsetC", c.offset());
+
         if(sizeMapping.globalAccumulation == 2)
             rv.args.append<typename TypedInputs::AlphaType>("alpha", inputs.alpha);
         else
@@ -788,9 +791,6 @@ namespace Tensile
             idx++;
         }
 
-        rv.args.append<uint32_t>("offsetD", d.offset());
-        rv.args.append<uint32_t>("offsetC", c.offset());
-
         if(sizeMapping.globalAccumulation == 1)
             rv.args.append<uint32_t>("gsu", 1);
         else
@@ -802,6 +802,11 @@ namespace Tensile
         return rv;
     }
 
+    bool ContractionSolution::canSolve(Problem const& problem, Hardware const& hardware) const
+    {
+        return (*problemPredicate)(problem) && (*hardwarePredicate)(hardware);
+    }
+
     template <typename TypedInputs>
     std::string ContractionSolution::outputConversionKernelName(Problem const&     problem,
                                                                 TypedInputs const& inputs,
@@ -1074,6 +1079,11 @@ namespace Tensile
         return size;
     }
 
+    float ContractionSolution::computeGranularity(float x)
+    {
+        return x / ceil(x);
+    }
+
     ContractionSolution::Granularities ContractionSolution::computeGranularities(
         Hardware const& hardware, double M, double N, double K, double NumBatches) const
     {
@@ -1101,8 +1111,8 @@ namespace Tensile
         granularities.numTiles0 = M / MT0;
         granularities.numTiles1 = N / MT1;
 
-        granularities.tile0Granularity = granularities.numTiles0 / ceil(granularities.numTiles0);
-        granularities.tile1Granularity = granularities.numTiles1 / ceil(granularities.numTiles1);
+        granularities.tile0Granularity = computeGranularity(granularities.numTiles0);
+        granularities.tile1Granularity = computeGranularity(granularities.numTiles1);
 
         granularities.tilesPerCu
             = (NumBatches * ceil(granularities.numTiles0) * ceil(granularities.numTiles1))
@@ -1111,8 +1121,7 @@ namespace Tensile
         granularities.totalTiles    = ceil(granularities.numTiles0) * ceil(granularities.numTiles1);
         granularities.natTilesPerCu = NumBatches * granularities.totalTiles / NumCUs;
         granularities.suTilesPerCu  = (granularities.totalTiles * GlobalSplitU) / NumCUs;
-        granularities.suCuGranularity
-            = granularities.suTilesPerCu / ceil(granularities.suTilesPerCu);
+        granularities.suCuGranularity = computeGranularity(granularities.suTilesPerCu);
 
         granularities.waveGranularity = std::min(
             1.00,
@@ -1132,7 +1141,7 @@ namespace Tensile
             = NumBatches * ceil(granularities.numTiles0) * ceil(granularities.numTiles1) / NumCUs;
         granularities.natCuGranularity = ceil(nat_tiles_per_cu) * ceil(nat_tiles_per_cu) / NumCUs;
 
-        granularities.cuGranularity = granularities.tilesPerCu / ceil(granularities.tilesPerCu);
+        granularities.cuGranularity = computeGranularity(granularities.tilesPerCu);
 
         granularities.totalGranularity
             = granularities.tile0Granularity * granularities.tile1Granularity
@@ -1355,5 +1364,4 @@ namespace Tensile
                       << " shiftPtrElemB=" << st.shiftPtrElemB << " depthUorMT0=" << st.depthUorMT0
                       << " depthUorMT1=" << st.depthUorMT1;
     }
-
 } // namespace Tensile
