@@ -36,7 +36,7 @@
 #include <Tensile/hip/HipUtils.hpp>
 
 //@TODO add alternative for windows
-#ifndef WIN32
+#ifndef _WIN32
 #include <glob.h>
 #endif
 #include <regex>
