@@ -118,10 +118,22 @@ def getAssemblyCodeObjectFiles(kernels, kernelWriterAssembly, outputPath):
               file.flush()
 
             args = [globalParameters['AssemblerPath'], '-target', 'amdgcn-amd-amdhsa', '-o', coFile, '@clangArgs.txt']
-            subprocess.check_call(args, cwd=asmDir)
+            # change to use  check_output to force windows cmd block util command finish
+            try:
+              out = subprocess.check_output(args, stderr=subprocess.STDOUT, cwd=asmDir)
+              print2(out)
+            except subprocess.CalledProcessError as err:
+              print(err.output)
+              raise
           else:
             args = kernelWriterAssembly.getLinkCodeObjectArgs(objectFiles, coFile)
-            subprocess.check_call(args, cwd=asmDir)
+            # change to use  check_output to force windows cmd block util command finish
+            try:
+              out = subprocess.check_output(args, stderr=subprocess.STDOUT, cwd=asmDir)
+              print2(out)
+            except subprocess.CalledProcessError as err:
+              print(err.output)
+              raise
 
           coFiles.append(coFile)
       else:
@@ -206,6 +218,12 @@ def buildSourceCodeObjectFile(CxxCompiler, outputPath, kernelFile):
 
       hipFlags += ['-I', outputPath]
 
+      # Add build-id for builds with rocm 5.3+
+      compilerVer = globalParameters['HipClangVersion'].split(".")[:2]
+      compilerVer = [int(c) for c in compilerVer]
+      if len(compilerVer) >= 2 and (compilerVer[0] > 5 or (compilerVer[0] == 5 and compilerVer[1] > 2)):
+        hipFlags += ["-Xoffload-linker", "--build-id"]
+
       launcher = shlex.split(os.environ.get('Tensile_CXX_COMPILER_LAUNCHER', ''))
 
       if os.name == "nt":
@@ -216,7 +234,13 @@ def buildSourceCodeObjectFile(CxxCompiler, outputPath, kernelFile):
 
       if globalParameters["PrintCodeCommands"]:
         print('hipcc:', ' '.join(compileArgs))
-      subprocess.check_call(compileArgs)
+      # change to use  check_output to force windows cmd block util command finish
+      try:
+        out = subprocess.check_output(compileArgs, stderr=subprocess.STDOUT)
+        print2(out)
+      except subprocess.CalledProcessError as err:
+        print(err.output)
+        raise
 
       # get hipcc version due to compatiblity reasons
       hipccver = globalParameters['HipClangVersion'].split(".")
@@ -225,8 +249,7 @@ def buildSourceCodeObjectFile(CxxCompiler, outputPath, kernelFile):
       # for hipclang 5.2 and above, clang offload bundler changes the way input/output files are specified
       inflag = "-inputs"
       outflag = "-outputs"
-      # clang 15.0.0 still wants inputs on windows so the hipcc version condition doesn't work
-      if os.name != "nt" and ((hipccMaj == 5 and hipccMin >= 2) or hipccMaj >= 6):
+      if (hipccMaj == 5 and hipccMin >= 2) or hipccMaj >= 6:
         inflag = "-input"
         outflag = "-output"
 
@@ -258,9 +281,12 @@ def buildSourceCodeObjectFile(CxxCompiler, outputPath, kernelFile):
                            "%s=%s" % (inflag, infile), "%s=%s" % (outflag, outfile), "-unbundle"]
               if globalParameters["PrintCodeCommands"]:
                 print(' '.join(bundlerArgs))
-              subprocess.check_call(bundlerArgs)
+              # change to use  check_output to force windows cmd block util command finish
+              out = subprocess.check_output(bundlerArgs, stderr=subprocess.STDOUT)
+              print2(out)
 
-      except subprocess.CalledProcessError:
+      except subprocess.CalledProcessError as err:
+        print(err.output)
         for i in range(len(archs)):
           outfile = os.path.join(buildPath, "{0}-000-{1}.hsaco".format(soFilename, archs[i]))
           coFilenames.append(os.path.split(outfile)[1])
@@ -269,7 +295,13 @@ def buildSourceCodeObjectFile(CxxCompiler, outputPath, kernelFile):
                          "%s=%s" % (inflag, infile), "%s=%s" % (outflag, outfile), "-unbundle"]
           if globalParameters["PrintCodeCommands"]:
             print(' '.join(bundlerArgs))
-          subprocess.check_call(bundlerArgs)
+          # change to use  check_output to force windows cmd block util command finish
+          try:
+            out = subprocess.check_output(bundlerArgs, stderr=subprocess.STDOUT)
+            print2(out)
+          except subprocess.CalledProcessError as err:
+            print(err.output)
+            raise
     else:
       raise RuntimeError("Unknown compiler {}".format(CxxCompiler))
 
@@ -310,13 +342,27 @@ def prepAsm(kernelWriterAssembly):
   Create and prepare the assembly directory  - called ONCE per output dir:
   """
   asmPath = ensurePath(os.path.join(globalParameters["WorkingPath"], "assembly") )
+  isa = globalParameters["CurrentISA"]
   assemblerFileName = os.path.join(asmPath, \
       "asm-new.%s"%("bat" if os.name=="nt" else "sh"))
   assemblerFile = open(assemblerFileName, "w")
   if os.name == "nt":
-    assemblerFile.write("echo Windows: Copying instead of Assembling\n")
-    assemblerFile.write("copy %1.s %1.o\n")
-    assemblerFile.write("copy %1.o %1.co\n")
+    assemblerFile.write("@echo off\n")
+    assemblerFile.write("set f=%1\n\n")
+    assemblerFile.write("set arg2=--wave64\n")
+    assemblerFile.write("if [%2] NEQ [] set arg2=%2\n\n")
+    assemblerFile.write("set /A wave=64\n")
+    assemblerFile.write("if %arg2% EQU --wave32 set /A wave=32\n\n")
+
+    assemblerFile.write("set h={gfxName}\n".format(gfxName = Common.gfxName(isa)))
+
+    cArgs32 = " ".join(kernelWriterAssembly.getCompileArgs("%f%.s", "%f%.o", isa=isa, wavefrontSize=32))
+    cArgs64 = " ".join(kernelWriterAssembly.getCompileArgs("%f%.s", "%f%.o", isa=isa, wavefrontSize=64))
+    lArgs   = " ".join(kernelWriterAssembly.getLinkCodeObjectArgs(["%f%.o"], "%f%.co"))
+
+    assemblerFile.write(f"if %wave% == 32 ({cArgs32}) else ({cArgs64})\n")
+    assemblerFile.write(f"{lArgs}\n")
+    assemblerFile.write( "copy %f%.co ..\..\..\library\%f%_%h%.co\n")
   else:
     assemblerFile.write("#!/bin/sh {log}\n".format(log = "-x" if globalParameters["PrintLevel"] >=2  else ""))
     assemblerFile.write("# usage: asm-new.sh kernelName(no extension) [--wave32]\n")
@@ -331,7 +377,6 @@ def prepAsm(kernelWriterAssembly):
     assemblerFile.write("fi\n")
 
 
-    isa = globalParameters["CurrentISA"]
     assemblerFile.write("h={gfxName}\n".format(gfxName = Common.gfxName(isa)))
 
     cArgs32 = kernelWriterAssembly.getCompileArgs("$f.s", "$f.o", isa=isa, wavefrontSize=32)
@@ -565,8 +610,12 @@ def writeSolutionsAndKernels(outputPath, CxxCompiler, problemTypes, solutions, k
   stop = time.time()
   print("# Kernel Building elapsed time = %.1f secs" % (stop-start))
 
+  Common.popWorkingPath() # outputPath.upper()
+
+  if globalParameters["CleanupBuildFiles"]:
+    shutil.rmtree(globalParameters["WorkingPath"])
+
   Common.popWorkingPath() # build_tmp
-  Common.popWorkingPath() # workingDir
 
   return codeObjectFiles
 
@@ -1131,7 +1180,7 @@ def TensileCreateLibrary():
   argParser.add_argument("RuntimeLanguage", help="Which runtime language?", choices=["OCL", "HIP", "HSA"])
   argParser.add_argument("--cxx-compiler",           dest="CxxCompiler",       choices=["hipcc"],       action="store", default="hipcc")
   argParser.add_argument("--cmake-cxx-compiler",     dest="CmakeCxxCompiler",  action="store")
-  argParser.add_argument("--code-object-version",    dest="CodeObjectVersion", choices=["V2", "V3"], action="store", default="V3")
+  argParser.add_argument("--code-object-version",    dest="CodeObjectVersion", choices=["default", "V4", "V5"], action="store")
   argParser.add_argument("--architecture",           dest="Architecture",      type=str, action="store", default="all", help="Supported archs: " + " ".join(architectureMap.keys()))
   argParser.add_argument("--merge-files",            dest="MergeFiles",        action="store_true")
   argParser.add_argument("--no-merge-files",         dest="MergeFiles",        action="store_false")
@@ -1167,7 +1216,8 @@ def TensileCreateLibrary():
   argParser.add_argument("--client-config", dest="ClientConfig", action="store_true",
                          help="Create client config for setting the library and code object files")
   argParser.add_argument("--global-parameters", nargs="+", type=splitExtraParameters, default=[])
-
+  argParser.add_argument("--ignore-asm-cap-cache", dest="IgnoreAsmCapCache", action="store_true", default=False,
+                         help="Ignore asm cap cache and derive the asm caps at runtime")
   args = argParser.parse_args()
 
   logicPath = args.LogicPath
@@ -1206,7 +1256,8 @@ def TensileCreateLibrary():
 
   arguments["CpuThreads"] = args.CpuThreads
   arguments["PrintLevel"] = args.PrintLevel
-
+  arguments["IgnoreAsmCapCache"] = args.IgnoreAsmCapCache
+  
   for key, value in args.global_parameters:
     arguments[key] = value
 
